#include "blc_array_tcp4_server.h"

#include <sys/types.h>
#include <sys/socket.h>
#include <sys/ioctl.h>
#include <netdb.h>
#include <unistd.h>
#include <netinet/tcp.h>
#include <vector>

using namespace std;

blc_array_tcp4_server::blc_array_tcp4_server(string const &port_name){
    int ret, error;
    struct addrinfo hints, *results;

    CLEAR(hints);
    hints.ai_flags =  AI_PASSIVE;
    hints.ai_family = PF_INET;
    hints.ai_socktype = SOCK_STREAM;
    hints.ai_protocol = IPPROTO_TCP;

    ret=getaddrinfo(nullptr, port_name.c_str(), &hints, &results);
    if (ret !=0) EXIT_ON_ERROR(gai_strerror(ret));
    if(results->ai_next != nullptr) EXIT_ON_ERROR("There is more than one possible address. It is not yet implemented.");

    SYSTEM_ERROR_CHECK(socket_fd = socket(results->ai_family, results->ai_socktype, 0), -1, nullptr);
    int flags =1;
    SYSTEM_ERROR_CHECK(setsockopt(socket_fd, IPPROTO_TCP, TCP_NODELAY, &flags, sizeof(flags)), -1, "socket %d", socket_fd); //Disable Naggle algorithm. Packet are send even if they are small
    SYSTEM_ERROR_CHECK(bind(socket_fd, const_cast<const struct sockaddr*>(results->ai_addr), results->ai_addrlen), -1, nullptr);
    freeaddrinfo(results);
    SYSTEM_ERROR_CHECK(listen(socket_fd, 1), -1, nullptr); /* Max one connection*/
}


blc_array_tcp4_server::blc_array_tcp4_server(string const &port_name, blc_array &array):blc_array_tcp4_server(port_name){
    blc_array::operator=(array);
}
    
/**
 If you do not want allocate data, use 'blc_array_tcp4_server(string const &port_name)' then '''server->def_array( type,  format, lengths);'''
 */
blc_array_tcp4_server::blc_array_tcp4_server(string const &port_name, uint32_t type, uint32_t format, vector<size_t> const &lengths):blc_array_tcp4_server(port_name){
    def_array(type, format, lengths);
    allocate();
}


int blc_array_tcp4_server::wait_connection(){
    struct sockaddr_storage client_storage;
    socklen_t client_addr_len=sizeof(client_storage);

    client_socket=accept(socket_fd, reinterpret_cast<struct sockaddr*>(&client_storage), &client_addr_len);

    if (client_socket==-1) return errno;
    SYSTEM_ERROR_CHECK(setsockopt(client_socket, SOL_SOCKET, SO_SNDBUF, &size, sizeof(size)), -1, nullptr);
    SYSTEM_ERROR_CHECK(setsockopt(client_socket, SOL_SOCKET, SO_RCVBUF, &size, sizeof(size)), -1, nullptr);
    send_properties();
    return 0;
}

void blc_array_tcp4_server::send_properties(){
    size_t length;
    vector<char> properties(1024);

    length=static_cast<size_t>(sprint_properties(properties.data(), properties.size()-1));
    if (length>=1024) EXIT_ON_ERROR(" length (%zu) too longe that the max (%zu)", length, properties.size());
    properties[length]=0;
    SYSTEM_ERROR_CHECK(write(client_socket, properties.data(), properties.size()), -1, nullptr);
}

int blc_array_tcp4_server::recv_data(){
    ssize_t ret;

    for (size_t read_nb=0; read_nb<size; read_nb+=static_cast<size_t>(ret)){
        ret=read(client_socket, chars+read_nb, size-read_nb);
        if (ret==-1) return errno;
    }
    return 0;
}

int blc_array_tcp4_server::send_data(){
    ssize_t ret;
    ret=write(client_socket, data, size);
    if (ret==-1) {
        if (errno==ECONNRESET) {
            SYSTEM_ERROR_CHECK(close(client_socket), -1, "client_socket %d", client_socket);
            client_socket=-1;
        }
        return errno;
    }
    else return 0;
}

blc_array_tcp4_server::~blc_array_tcp4_server(){
    SYSTEM_ERROR_CHECK(close(socket_fd), -1, nullptr);
}

